import { SvelteSet } from "svelte/reactivity";

export interface Workout {
	id?: number;
	date: string;
	type: string;
	duration_minutes: number;
}


export class WorkoutLog {
	#types = new SvelteSet<string>(['jogging', 'crosstrainer', 'bicycle', 'crossfit']);
	#workouts = $state<Workout[]>([]);

	get workouts(): Workout[] {
		return Array.from(this.#workouts);
	}

	get types() {
		return Array.from(this.#types.values());
	}

	#load_from_storage() {
		if (typeof localStorage === 'undefined') {
			return;
		}

		const storage = localStorage.getItem('workout_log');

		if (!storage) {
			return
		}
		const data: {
			types: string[],
			workouts: Workout[],
		} = JSON.parse(storage);

		this.#types = new SvelteSet<string>([...data.types]);
		this.#workouts.length = 0;
		this.#workouts.push(...data.workouts);
	}

	#save_to_storage() {
		if (typeof localStorage !== 'undefined') {

			localStorage.setItem('workout_log', this.#to_json());
		}
	}
	#to_json(): string {
		return JSON.stringify({
			types: this.types,
			workouts: this.workouts,
		});
	}
	constructor() {
		this.#load_from_storage();
	}

	add_workout(workout: Workout) {


		while (this.#workouts.indexOf(workout) !== -1) {
			if (!workout.id) {
				workout.id = 1;
				continue
			}
			workout.id += 5;
		}
		this.#workouts.push(workout);
		this.#types.add(workout.type.toLowerCase())
		this.#save_to_storage();
	}

	remove_workout(workout: Workout) {
		//this.#workouts.delete(workout);
		this.#workouts = this.#workouts.filter(i => i !== workout);
		this.#save_to_storage();
	}

	remove_type(type: string) {
		this.#types.delete(type);
		this.#save_to_storage();
	}


}

export default WorkoutLog;
